@@ -25,49 +25,55 @@
  */
 
 #include <config.h>
-
 #include "nautilus-application.h"
 
-#include "nautilus-dbus-manager.h"
-#include "nautilus-file-undo-manager.h"
-#include "nautilus-freedesktop-dbus.h"
-#include "nautilus-image-properties-page.h"
-#include "nautilus-previewer.h"
-#include "nautilus-progress-persistence-handler.h"
-#include "nautilus-self-check-functions.h"
-#include "nautilus-shell-search-provider.h"
-#include "nautilus-window.h"
-#include "nautilus-window-slot.h"
-#include "nautilus-preferences-window.h"
-
-#include "nautilus-directory-private.h"
-#include "nautilus-file-utilities.h"
-#include "nautilus-file-operations.h"
-#include "nautilus-global-preferences.h"
-#include "nautilus-lib-self-check-functions.h"
-#include "nautilus-module.h"
-#include "nautilus-profile.h"
-#include "nautilus-signaller.h"
-#include "nautilus-ui-utilities.h"
-#include <libnautilus-extension/nautilus-menu-provider.h>
+#include <eel/eel-gtk-extensions.h>
+#include <eel/eel-stock-dialogs.h>
+#include <fcntl.h>
+#include <gdk/gdk.h>
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+#include <gio/gio.h>
+#include <glib/gi18n.h>
+#include <glib/gstdio.h>
+#include <gtk/gtk.h>
+#include <nautilus-extension.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #define DEBUG_FLAG NAUTILUS_DEBUG_APPLICATION
 #include "nautilus-debug.h"
 
-#ifdef HAVE_UNITY
-#include "src/unity-bookmarks-handler.h"
-#endif
+#include "nautilus-bookmark-list.h"
+#include "nautilus-dbus-manager.h"
+#include "nautilus-directory-private.h"
+#include "nautilus-file.h"
+#include "nautilus-files-view.h"
+#include "nautilus-file-operations.h"
+#include "nautilus-file-undo-manager.h"
+#include "nautilus-file-utilities.h"
+#include "nautilus-freedesktop-dbus.h"
+#include "nautilus-global-preferences.h"
+#include "nautilus-icon-info.h"
+#include "nautilus-lib-self-check-functions.h"
+#include "nautilus-module.h"
+#include "nautilus-preferences-window.h"
+#include "nautilus-previewer.h"
+#include "nautilus-profile.h"
+#include "nautilus-progress-persistence-handler.h"
+#include "nautilus-self-check-functions.h"
+#include "nautilus-shell-search-provider.h"
+#include "nautilus-signaller.h"
+#include "nautilus-tag-manager.h"
+#include "nautilus-ui-utilities.h"
+#include "nautilus-view.h"
+#include "nautilus-window-slot.h"
+#include "nautilus-window.h"
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
-#include <gio/gio.h>
-#include <eel/eel-gtk-extensions.h>
-#include <eel/eel-stock-dialogs.h>
-#include <gdk/gdkx.h>
-#include <gtk/gtk.h>
+#if HAVE_UNITY
+#include "unity-bookmarks-handler.h"
+#endif
 
 typedef struct
 {
@@ -84,6 +90,11 @@ typedef struct
     GHashTable *notifications;
 
     NautilusFileUndoManager *undo_manager;
+
+    NautilusTagManager *tag_manager;
+    GCancellable *tag_manager_cancellable;
+
+    guint previewer_selection_id;
 } NautilusApplicationPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (NautilusApplication, nautilus_application, GTK_TYPE_APPLICATION);
@@ -192,7 +203,7 @@ check_required_directories (NautilusApplication *self)
                                              directories_as_string->str);
         }
 
-        dialog = eel_show_error_dialog (error_string, detail_string, NULL);
+        dialog = show_dialog (error_string, detail_string, NULL, GTK_MESSAGE_ERROR);
         /* We need the main event loop so the user has a chance to see the dialog. */
         gtk_application_add_window (GTK_APPLICATION (self),
                                     GTK_WINDOW (dialog));
@@ -240,17 +251,15 @@ NautilusWindow *
 nautilus_application_create_window (NautilusApplication *self,
                                     GdkScreen           *screen)
 {
-    NautilusApplicationPrivate *priv;
     NautilusWindow *window;
-    char *geometry_string;
     gboolean maximized;
-    gint n_windows;
+    g_autoptr (GVariant) default_size = NULL;
+    gint default_width = 0;
+    gint default_height = 0;
 
     g_return_val_if_fail (NAUTILUS_IS_APPLICATION (self), NULL);
     nautilus_profile_start (NULL);
 
-    priv = nautilus_application_get_instance_private (self);
-    n_windows = g_list_length (priv->windows);
     window = nautilus_window_new (screen);
 
     maximized = g_settings_get_boolean
@@ -263,24 +272,23 @@ nautilus_application_create_window (NautilusApplication *self,
     {
         gtk_window_unmaximize (GTK_WINDOW (window));
     }
+    default_size = g_settings_get_value (nautilus_window_state,
+                                         NAUTILUS_WINDOW_STATE_INITIAL_SIZE);
 
-    geometry_string = g_settings_get_string
-                          (nautilus_window_state, NAUTILUS_WINDOW_STATE_GEOMETRY);
-    if (geometry_string != NULL &&
-        geometry_string[0] != 0)
+    g_variant_get (default_size, "(ii)", &default_width, &default_height);
+
+    gtk_window_set_default_size (GTK_WINDOW (window),
+                                 MAX (NAUTILUS_WINDOW_MIN_WIDTH, default_width),
+                                 MAX (NAUTILUS_WINDOW_MIN_HEIGHT, default_height));
+
+    if (g_strcmp0 (PROFILE, "") != 0)
     {
-        /* Ignore saved window position if another window is already showing.
-         * That way the two windows wont appear at the exact same
-         * location on the screen.
-         */
-        eel_gtk_window_set_initial_geometry_from_string
-            (GTK_WINDOW (window),
-            geometry_string,
-            NAUTILUS_WINDOW_MIN_WIDTH,
-            NAUTILUS_WINDOW_MIN_HEIGHT,
-            n_windows > 0);
+        GtkStyleContext *style_context;
+
+        style_context = gtk_widget_get_style_context (GTK_WIDGET (window));
+
+        gtk_style_context_add_class (style_context, "devel");
     }
-    g_free (geometry_string);
 
     DEBUG ("Creating a new navigation window");
     nautilus_profile_end (NULL);
@@ -340,20 +348,6 @@ get_window_slot_for_location (NautilusApplication *self,
     return slot;
 }
 
-static void
-new_window_show_callback (GtkWidget *widget,
-                          gpointer   user_data)
-{
-    NautilusWindow *window;
-
-    window = NAUTILUS_WINDOW (user_data);
-    nautilus_window_close (window);
-
-    g_signal_handlers_disconnect_by_func (widget,
-                                          G_CALLBACK (new_window_show_callback),
-                                          user_data);
-}
-
 void
 nautilus_application_open_location_full (NautilusApplication     *self,
                                          GFile                   *location,
@@ -466,23 +460,6 @@ real_open_location_full (NautilusApplication     *self,
 
     g_assert (target_window != NULL);
 
-    /* close the current window if the flags say so */
-    if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND) != 0)
-    {
-        if (gtk_widget_get_visible (GTK_WIDGET (target_window)))
-        {
-            nautilus_window_close (active_window);
-        }
-        else
-        {
-            g_signal_connect_object (target_window,
-                                     "show",
-                                     G_CALLBACK (new_window_show_callback),
-                                     active_window,
-                                     G_CONNECT_AFTER);
-        }
-    }
-
     /* Application is the one that manages windows, so this flag shouldn't use
      * it anymore by any client */
     flags &= ~NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
@@ -616,6 +593,11 @@ nautilus_application_finalize (GObject *object)
 
     g_clear_object (&priv->undo_manager);
 
+    g_clear_object (&priv->tag_manager);
+
+    g_cancellable_cancel (priv->tag_manager_cancellable);
+    g_clear_object (&priv->tag_manager_cancellable);
+
     G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
 }
 
@@ -742,7 +724,19 @@ action_clone_window (GSimpleAction *action,
     }
     else
     {
+        /* If the user happens to fall asleep while holding ctrl-n, or very
+         * unfortunately opens a new window at a remote location, the current
+         * location will be null, leading to criticals and/or failed assertions.
+         *
+         * Another sad thing is that checking if the view/slot is loading will
+         * not work, as the loading process only really begins after the attributes
+         * for the file have been fetched.
+         */
         location = nautilus_window_slot_get_location (active_slot);
+        if (location == NULL)
+        {
+            location = nautilus_window_slot_get_pending_location (active_slot);
+        }
     }
 
     nautilus_application_open_location_full (NAUTILUS_APPLICATION (application), location,
@@ -882,7 +876,7 @@ variant_set_mapping (const GValue       *value,
     return g_value_get_variant (value);
 }
 
-static GActionEntry app_entries[] =
+const static GActionEntry app_entries[] =
 {
     { "new-window", action_new_window, NULL, NULL, NULL },
     { "clone-window", action_clone_window, NULL, NULL, NULL },
@@ -898,23 +892,12 @@ static GActionEntry app_entries[] =
 static void
 nautilus_init_application_actions (NautilusApplication *app)
 {
-    const gchar *debug_no_app_menu;
     GAction *sidebar_action;
 
     g_action_map_add_action_entries (G_ACTION_MAP (app),
                                      app_entries, G_N_ELEMENTS (app_entries),
                                      app);
 
-    debug_no_app_menu = g_getenv ("NAUTILUS_DEBUG_NO_APP_MENU");
-    if (debug_no_app_menu)
-    {
-        DEBUG ("Disabling app menu GtkSetting as requested...");
-        g_object_set (gtk_settings_get_default (),
-                      "gtk-shell-shows-app-menu", FALSE,
-                      NULL);
-    }
-
-
 
     sidebar_action = g_action_map_lookup_action (G_ACTION_MAP (app),
                                                  "show-hide-sidebar");
@@ -931,38 +914,16 @@ nautilus_init_application_actions (NautilusApplication *app)
                                   variant_set_mapping,
                                   NULL, NULL);
 
-    nautilus_application_set_accelerator (G_APPLICATION (app), "app.show-hide-sidebar", "F9");
+    nautilus_application_set_accelerator (G_APPLICATION (app),
+                                          "app.clone-window", "<Primary>n");
+    nautilus_application_set_accelerator (G_APPLICATION (app),
+                                          "app.help", "F1");
+    nautilus_application_set_accelerator (G_APPLICATION (app),
+                                          "app.quit", "<Primary>q");
+    nautilus_application_set_accelerator (G_APPLICATION (app),
+                                          "app.show-hide-sidebar", "F9");
 }
 
-const GOptionEntry options[] =
-{
-        #ifndef NAUTILUS_OMIT_SELF_CHECK
-    { "check", 'c', 0, G_OPTION_ARG_NONE, NULL,
-      N_("Perform a quick set of self-check tests."), NULL },
-        #endif
-    /* dummy, only for compatibility reasons */
-    { "browser", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
-      NULL, NULL },
-    { "no-desktop", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
-      NULL, NULL },
-    /* ditto */
-    { "geometry", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, NULL,
-      N_("Create the initial window with the given geometry."), N_("GEOMETRY") },
-    { "version", '\0', 0, G_OPTION_ARG_NONE, NULL,
-      N_("Show the version of the program."), NULL },
-    { "new-window", 'w', 0, G_OPTION_ARG_NONE, NULL,
-      N_("Always open a new window for browsing specified URIs"), NULL },
-    { "no-default-window", 'n', 0, G_OPTION_ARG_NONE, NULL,
-      N_("Only create windows for explicitly specified URIs."), NULL },
-    { "quit", 'q', 0, G_OPTION_ARG_NONE, NULL,
-      N_("Quit Nautilus."), NULL },
-    { "select", 's', 0, G_OPTION_ARG_NONE, NULL,
-      N_("Select specified URI in parent folder."), NULL },
-    { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, NULL, NULL, N_("[URI…]") },
-
-    { NULL }
-};
-
 static void
 nautilus_application_activate (GApplication *app)
 {
@@ -1009,7 +970,20 @@ nautilus_application_handle_file_args (NautilusApplication *self,
                 file = g_file_new_for_commandline_arg_and_cwd (remaining[idx], cwd);
                 g_free (cwd);
             }
-            g_ptr_array_add (file_array, file);
+
+            if (nautilus_is_search_directory (file))
+            {
+                g_autofree char *error_string = NULL;
+                error_string = g_strdup_printf (_("“%s” is an internal protocol. "
+                                                  "Opening this location directly is not supported."),
+                                                EEL_SEARCH_URI);
+
+                g_printerr ("%s\n", error_string);
+            }
+            else
+            {
+                g_ptr_array_add (file_array, file);
+            }
         }
     }
     else if (g_variant_dict_contains (options, "new-window"))
@@ -1096,6 +1070,36 @@ out:
 static void
 nautilus_application_init (NautilusApplication *self)
 {
+    static const GOptionEntry options[] =
+    {
+#ifndef NAUTILUS_OMIT_SELF_CHECK
+        { "check", 'c', 0, G_OPTION_ARG_NONE, NULL,
+          N_("Perform a quick set of self-check tests."), NULL },
+#endif
+        { "version", '\0', 0, G_OPTION_ARG_NONE, NULL,
+          N_("Show the version of the program."), NULL },
+        { "new-window", 'w', 0, G_OPTION_ARG_NONE, NULL,
+          N_("Always open a new window for browsing specified URIs"), NULL },
+        { "quit", 'q', 0, G_OPTION_ARG_NONE, NULL,
+          N_("Quit Nautilus."), NULL },
+        { "select", 's', 0, G_OPTION_ARG_NONE, NULL,
+          N_("Select specified URI in parent folder."), NULL },
+        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, NULL, NULL, N_("[URI…]") },
+
+        /* The following are old options which have no effect anymore. We keep
+         * them around for compatibility reasons, e.g. not breaking old scripts.
+         */
+        { "browser", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
+          NULL, NULL },
+        { "geometry", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, NULL,
+          NULL, NULL },
+        { "no-default-window", 'n', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
+          NULL, NULL },
+        { "no-desktop", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
+          NULL, NULL },
+
+        { NULL }
+    };
     NautilusApplicationPrivate *priv;
 
     priv = nautilus_application_get_instance_private (self);
@@ -1107,6 +1111,11 @@ nautilus_application_init (NautilusApplication *self)
 
     priv->undo_manager = nautilus_file_undo_manager_new ();
 
+    priv->tag_manager_cancellable = g_cancellable_new ();
+    priv->tag_manager = nautilus_tag_manager_get ();
+    nautilus_tag_manager_set_cancellable (priv->tag_manager,
+                                          priv->tag_manager_cancellable);
+
     g_application_add_main_option_entries (G_APPLICATION (self), options);
 
     nautilus_ensure_extension_points ();
@@ -1219,6 +1228,59 @@ nautilus_application_withdraw_notification (NautilusApplication *self,
     g_application_withdraw_notification (G_APPLICATION (self), notification_id);
 }
 
+static void
+update_previewer_selection (NautilusApplication *self,
+                            NautilusWindow      *window)
+{
+    GtkWindow *gtk_window;
+    NautilusWindowSlot *slot;
+    NautilusView *view;
+    GList *selection;
+
+    gtk_window = gtk_application_get_active_window (GTK_APPLICATION (self));
+    if (!NAUTILUS_IS_WINDOW (gtk_window))
+    {
+        return;
+    }
+
+    if (NAUTILUS_WINDOW (gtk_window) != window)
+    {
+        return;
+    }
+
+    slot = nautilus_window_get_active_slot (window);
+    if (slot == NULL)
+    {
+        return;
+    }
+
+    view = nautilus_window_slot_get_current_view (slot);
+    if (!NAUTILUS_IS_FILES_VIEW (view))
+    {
+        return;
+    }
+
+    selection = nautilus_window_slot_get_selection (slot);
+    if (selection != NULL)
+    {
+        nautilus_files_view_preview_update (NAUTILUS_FILES_VIEW (view), selection);
+    }
+}
+
+static void
+on_application_active_window_changed (NautilusApplication *self,
+                                      GParamSpec          *pspec,
+                                      gpointer             user_data)
+{
+    GtkWindow *window;
+
+    window = gtk_application_get_active_window (GTK_APPLICATION (self));
+    if (NAUTILUS_IS_WINDOW (window))
+    {
+        update_previewer_selection (self, NAUTILUS_WINDOW (window));
+    }
+}
+
 static void
 on_application_shutdown (GApplication *application,
                          gpointer      user_data)
@@ -1243,6 +1305,20 @@ on_application_shutdown (GApplication *application,
     nautilus_icon_info_clear_caches ();
 }
 
+static void
+icon_theme_changed_callback (GtkIconTheme *icon_theme,
+                             gpointer      user_data)
+{
+    /* Clear all pixmap caches as the icon => pixmap lookup changed */
+    nautilus_icon_info_clear_caches ();
+
+    /* Tell the world that icons might have changed. We could invent a narrower-scope
+     * signal to mean only "thumbnails might have changed" if this ends up being slow
+     * for some reason.
+     */
+    emit_change_signals_for_all_files_in_all_directories ();
+}
+
 void
 nautilus_application_startup_common (NautilusApplication *self)
 {
@@ -1258,16 +1334,13 @@ nautilus_application_startup_common (NautilusApplication *self)
      */
     G_APPLICATION_CLASS (nautilus_application_parent_class)->startup (G_APPLICATION (self));
 
-    gtk_window_set_default_icon_name ("org.gnome.Nautilus");
+    gtk_window_set_default_icon_name (APPLICATION_ID);
 
     setup_theme_extensions ();
 
     /* initialize preferences and create the global GSettings objects */
     nautilus_global_preferences_init ();
 
-    /* register property pages */
-    nautilus_image_properties_page_register ();
-
     /* initialize nautilus modules */
     nautilus_profile_start ("Modules");
     nautilus_module_setup ();
@@ -1288,7 +1361,13 @@ nautilus_application_startup_common (NautilusApplication *self)
 
     nautilus_profile_end (NULL);
 
+    g_signal_connect (self, "notify::active-window", G_CALLBACK (on_application_active_window_changed), NULL);
     g_signal_connect (self, "shutdown", G_CALLBACK (on_application_shutdown), NULL);
+
+    g_signal_connect_object (gtk_icon_theme_get_default (),
+                             "changed",
+                             G_CALLBACK (icon_theme_changed_callback),
+                             NULL, 0);
 }
 
 static void
@@ -1306,7 +1385,7 @@ nautilus_application_startup (GApplication *app)
 
     nautilus_profile_end (NULL);
 
-#ifdef HAVE_UNITY
+#if HAVE_UNITY
     unity_bookmarks_handler_initialize ();
 #endif
 }
@@ -1333,6 +1412,8 @@ nautilus_application_dbus_register (GApplication     *app,
         return FALSE;
     }
 
+    priv->previewer_selection_id = nautilus_previewer_connect_selection_event (connection);
+
     return TRUE;
 }
 
@@ -1356,10 +1437,17 @@ nautilus_application_dbus_unregister (GApplication    *app,
         nautilus_shell_search_provider_unregister (priv->search_provider);
         g_clear_object (&priv->search_provider);
     }
+
+    if (priv->previewer_selection_id != 0)
+    {
+        nautilus_previewer_disconnect_selection_event (connection,
+                                                       priv->previewer_selection_id);
+        priv->previewer_selection_id = 0;
+    }
 }
 
 void
-nautilus_application_update_dbus_opened_locations (NautilusApplication *self)
+update_dbus_opened_locations (NautilusApplication *self)
 {
     NautilusApplicationPrivate *priv;
     gint i;
@@ -1369,11 +1457,14 @@ nautilus_application_update_dbus_opened_locations (NautilusApplication *self)
     gchar **locations_array;
     NautilusWindow *window;
     GFile *location;
+    const gchar *dbus_object_path = NULL;
 
-    g_return_if_fail (NAUTILUS_IS_APPLICATION (self));
-
+    g_autoptr (GVariant) windows_to_locations = NULL;
     GVariantBuilder b;
     GVariantBuilder wb;
+    GVariantBuilder windows_to_locations_builder;
+
+    g_return_if_fail (NAUTILUS_IS_APPLICATION (self));
 
     g_variant_builder_init (&b, G_VARIANT_TYPE ("a{uas}"));
 
@@ -1386,13 +1477,24 @@ nautilus_application_update_dbus_opened_locations (NautilusApplication *self)
         return;
     }
 
+    dbus_object_path = g_application_get_dbus_object_path (G_APPLICATION (self));
+
+    g_return_if_fail (dbus_object_path);
+
+    g_variant_builder_init (&windows_to_locations_builder, G_VARIANT_TYPE ("a{sas}"));
+
     for (l = priv->windows; l != NULL; l = l->next)
     {
+        guint32 id;
+        g_autofree gchar *path = NULL;
+        NautilusWindowSlot *active_slot;
+        GVariantBuilder locations_in_window_builder;
+
         window = l->data;
 
         g_variant_builder_init (&wb, G_VARIANT_TYPE ("as"));
 
-        NautilusWindowSlot *active_slot = nautilus_window_get_active_slot (window);
+        active_slot = nautilus_window_get_active_slot (window);
 
         if (active_slot != NULL)
         {
@@ -1404,7 +1506,9 @@ nautilus_application_update_dbus_opened_locations (NautilusApplication *self)
                     g_variant_builder_add (&wb, "s", active_uri);
                     g_free (active_uri);
                 }
-         }
+        }
+
+        g_variant_builder_init (&locations_in_window_builder, G_VARIANT_TYPE ("as"));
 
         for (sl = nautilus_window_get_slots (window); sl; sl = sl->next)
         {
@@ -1413,14 +1517,19 @@ nautilus_application_update_dbus_opened_locations (NautilusApplication *self)
 
             if (location != NULL)
             {
-                gchar *uri = g_file_get_uri (location);
+                gchar *uri;
+                GList *found;
+
+                uri = g_file_get_uri (location);
 
                 if (slot != active_slot)
                 {
                     g_variant_builder_add (&wb, "s", uri);
                 }
 
-                GList *found = g_list_find_custom (locations, uri, (GCompareFunc) g_strcmp0);
+                found = g_list_find_custom (locations, uri, (GCompareFunc) g_strcmp0);
+
+                g_variant_builder_add (&locations_in_window_builder, "s", uri);
 
                 if (!found)
                 {
@@ -1434,25 +1543,32 @@ nautilus_application_update_dbus_opened_locations (NautilusApplication *self)
             }
         }
 
-   if (gtk_widget_get_realized (GTK_WIDGET (l->data)))
-   {
-       guint32 window_id = 0;
-       GdkWindow *gdk_win = gtk_widget_get_window (GTK_WIDGET (l->data));
+        if (gtk_widget_get_realized (GTK_WIDGET (l->data)))
+        {
+           guint32 window_id;
+           GdkWindow *gdk_win;
+
+           window_id = 0;
+           gdk_win = gtk_widget_get_window (GTK_WIDGET (l->data));
 #ifdef GDK_WINDOWING_X11
-       if (GDK_IS_X11_WINDOW (gdk_win))
-       {
-           window_id = gdk_x11_window_get_xid (gdk_win);
-       }
-       else
-       {
-           window_id = GPOINTER_TO_UINT(gdk_win);
-       }
+           if (GDK_IS_X11_WINDOW (gdk_win))
+           {
+               window_id = gdk_x11_window_get_xid (gdk_win);
+           }
+           else
+           {
+               window_id = GPOINTER_TO_UINT(gdk_win);
+           }
 #endif
-       g_variant_builder_add (&b, "{uas}", window_id, &wb);
-    }
+           g_variant_builder_add (&b, "{uas}", window_id, &wb);
+        }
 
-    g_variant_builder_clear (&wb);
+        g_variant_builder_clear (&wb);
 
+        id = gtk_application_window_get_id (GTK_APPLICATION_WINDOW (window));
+        path = g_strdup_printf ("%s/window/%u", dbus_object_path, id);
+        g_variant_builder_add (&windows_to_locations_builder, "{sas}", path, &locations_in_window_builder);
+        g_variant_builder_clear (&locations_in_window_builder);
     }
 
     locations_array = g_new (gchar *, locations_size + 1);
@@ -1471,6 +1587,10 @@ nautilus_application_update_dbus_opened_locations (NautilusApplication *self)
     nautilus_freedesktop_dbus_set_open_locations_xids (priv->fdb_manager,
                                                        g_variant_builder_end (&b));
 
+    windows_to_locations = g_variant_ref_sink (g_variant_builder_end (&windows_to_locations_builder));
+    nautilus_freedesktop_dbus_set_open_windows_with_locations (priv->fdb_manager,
+                                                               windows_to_locations);
+
     g_free (locations_array);
     g_list_free_full (locations, g_free);
 }
@@ -1480,14 +1600,7 @@ on_slot_location_changed (NautilusWindowSlot  *slot,
                           GParamSpec          *pspec,
                           NautilusApplication *self)
 {
-    nautilus_application_update_dbus_opened_locations (self);
-}
-
-static void
-on_slot_active_changed (NautilusWindowSlot *slot,
-                        NautilusApplication *self)
-{
-    nautilus_application_update_dbus_opened_locations (self);
+    update_dbus_opened_locations (self);
 }
 
 static void
@@ -1497,7 +1610,7 @@ on_slot_added (NautilusWindow      *window,
 {
     if (nautilus_window_slot_get_location (slot))
     {
-        nautilus_application_update_dbus_opened_locations (self);
+        update_dbus_opened_locations (self);
     }
 
     g_signal_connect (slot, "notify::location", G_CALLBACK (on_slot_location_changed), self);
@@ -1508,16 +1621,23 @@ on_slot_removed (NautilusWindow      *window,
                  NautilusWindowSlot  *slot,
                  NautilusApplication *self)
 {
-    nautilus_application_update_dbus_opened_locations (self);
+    update_dbus_opened_locations (self);
 
-    g_signal_handlers_disconnect_by_func (slot, on_slot_active_changed, self);
+    g_signal_handlers_disconnect_by_func (slot, on_slot_location_changed, self);
+}
+
+static void
+on_active_selection_changed (NautilusWindow      *window,
+                             NautilusApplication *self)
+{
+    update_previewer_selection (self, window);
 }
 
 static void
 on_realize (GtkWidget           *widget,
             NautilusApplication *self)
 {
-    nautilus_application_update_dbus_opened_locations (self);
+    update_dbus_opened_locations (self);
 }
 
 static void
@@ -1536,6 +1656,7 @@ nautilus_application_window_added (GtkApplication *app,
         g_signal_connect (window, "realize", G_CALLBACK (on_realize), app);
         g_signal_connect (window, "slot-added", G_CALLBACK (on_slot_added), app);
         g_signal_connect (window, "slot-removed", G_CALLBACK (on_slot_removed), app);
+        g_signal_connect (window, "active-selection-changed", G_CALLBACK (on_active_selection_changed), app);
     }
 }
 
@@ -1556,6 +1677,7 @@ nautilus_application_window_removed (GtkApplication *app,
         g_signal_handlers_disconnect_by_func (window, on_realize, app);
         g_signal_handlers_disconnect_by_func (window, on_slot_added, app);
         g_signal_handlers_disconnect_by_func (window, on_slot_removed, app);
+        g_signal_handlers_disconnect_by_func (window, on_active_selection_changed, app);
     }
 
     /* if this was the last window, close the previewer */
@@ -1613,7 +1735,7 @@ NautilusApplication *
 nautilus_application_new (void)
 {
     return g_object_new (NAUTILUS_TYPE_APPLICATION,
-                         "application-id", "org.gnome.Nautilus",
+                         "application-id", APPLICATION_ID,
                          "flags", G_APPLICATION_HANDLES_COMMAND_LINE | G_APPLICATION_HANDLES_OPEN,
                          "inactivity-timeout", 12000,
                          NULL);
