@@ -26,7 +26,7 @@
 #include <gio/gio.h>
 #include <string.h>
 #include <glib/gi18n.h>
-#include <gdk/gdkx.h>
+#include <gdk/gdk.h>
 
 #include "nautilus-file.h"
 #include "nautilus-file-utilities.h"
@@ -549,7 +549,6 @@ static void
 result_metas_data_free (ResultMetasData *data)
 {
     g_clear_pointer (&data->handle, nautilus_file_list_cancel_call_when_ready);
-
     g_clear_object (&data->self);
     g_clear_object (&data->invocation);
     g_strfreev (data->uris);
@@ -582,35 +581,45 @@ result_metas_return_from_cache (ResultMetasData *data)
                                            g_variant_new ("(aa{sv})", &builder));
 }
 
+static void
+result_metas_return_empty (ResultMetasData *data)
+{
+    g_clear_pointer (&data->uris, g_strfreev);
+    result_metas_return_from_cache (data);
+    result_metas_data_free (data);
+}
+
 static void
 cancel_result_meta_requests (NautilusShellSearchProvider *self,
                              GDBusMethodInvocation       *invocation)
 {
-    GList *l;
-    GList *to_remove = NULL;
+    ResultMetasData *data;
 
     g_debug ("*** Cancel Results Meta requests");
 
-    for (l = self->metas_requests; l; l = l->next)
+    if (invocation == NULL)
     {
-        ResultMetasData *data = l->data;
+        g_list_free_full (self->metas_requests,
+                          (GDestroyNotify) result_metas_return_empty);
+        self->metas_requests = NULL;
+    }
+    else
+    {
+        GList *l, *next = NULL;
 
-        if (invocation == NULL ||
-            g_strcmp0 (g_dbus_method_invocation_get_sender (data->invocation),
-                       g_dbus_method_invocation_get_sender (invocation)) == 0)
+        for (l = self->metas_requests; l != NULL; l = next)
         {
-            g_clear_pointer (&data->uris, g_strfreev);
-            result_metas_return_from_cache (data);
-            to_remove = g_list_prepend (to_remove, data);
+            next = l->next;
+            data = l->data;
+
+            if (g_strcmp0 (g_dbus_method_invocation_get_sender (data->invocation),
+                           g_dbus_method_invocation_get_sender (invocation)) == 0)
+            {
+                result_metas_return_empty (data);
+                self->metas_requests = g_list_delete_link (self->metas_requests, l);
+            }
         }
     }
-
-    for (l = to_remove; l; l = l->next)
-    {
-        self->metas_requests = g_list_remove (self->metas_requests, l->data);
-    }
-
-    g_list_free (to_remove);
 }
 
 static void
@@ -775,20 +784,24 @@ handle_launch_search (NautilusShellSearchProvider2  *skeleton,
 {
     GApplication *app = g_application_get_default ();
     g_autoptr (NautilusQuery) query = shell_query_new (terms);
+    NautilusQueryRecursive recursive = location_settings_search_get_recursive ();
 
-    if (location_settings_search_get_recursive () == NAUTILUS_QUERY_RECURSIVE_NEVER)
+    /*
+     * If no recursive search is enabled, we still want to be able to
+     * show the same results we presented in the overview when nautilus
+     * is explicitly launched to access to more results, and thus we perform
+     * a query showing results coming from index-based search engines.
+     * Otherwise we respect the global setting for recursivity.
+     */
+    if (recursive == NAUTILUS_QUERY_RECURSIVE_NEVER)
     {
-        /*
-         * If no recursive search is enabled, we still want to be able to
-         * show the same results we presented in the overview when nautilus
-         * is explicitly launched to access to more results, and thus we perform
-         * a query showing results coming from index-based search engines.
-         * Otherwise we just respect the user settings.
-         * See: https://gitlab.gnome.org/GNOME/nautilus/merge_requests/249
-         */
         nautilus_query_set_recursive (query,
                                       NAUTILUS_QUERY_RECURSIVE_INDEXED_ONLY);
     }
+    else
+    {
+        nautilus_query_set_recursive (query, recursive);
+    }
 
     nautilus_application_search (NAUTILUS_APPLICATION (app), query);
 
@@ -820,7 +833,9 @@ handle_xubuntu_cancel (NautilusShellSearchProvider2 *skeleton,
     return TRUE;
 }
 
-  static void search_provider_dispose(GObject * obj) {
+static void
+search_provider_dispose (GObject *obj)
+{
     NautilusShellSearchProvider *self = NAUTILUS_SHELL_SEARCH_PROVIDER (obj);
 
     g_clear_object (&self->skeleton);
@@ -875,7 +890,7 @@ nautilus_shell_search_provider_register (NautilusShellSearchProvider  *self,
 {
     return g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (self->skeleton),
                                              connection,
-                                             "/org/gnome/Nautilus/SearchProvider", error);
+                                             "/org/gnome/Nautilus" PROFILE "/SearchProvider", error);
 }
 
 void
